#!/bin/sh
# This script produces a report of synspam
# prints:
# 1/ top ten spammers AS (autonomous system)
# 2/ average accept score
# 3/ average reject score

export LC_ALL=C
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
REPORT="/tmp/synspam-report"
SASTATS="/usr/local/bin/sa-stats-ng.pl"
DAY=$(date --date '1 day ago' +"%b %d" | sed -e 's/0\([1-9]\)/\ \1/')
LOGS="/tmp/mail_logs"
ACCEPT_COUNT=$(cat  /var/log/daemon.log /var/log/daemon.log.? |grep -c "$DAY.*synspam\:.*accept.*")
ACCEPT_SCORES_FILE="/tmp/synspam_accept"
REJECT_COUNT=$(cat  /var/log/daemon.log /var/log/daemon.log.? |grep -c "$DAY.*synspam\:.*reject.*")
REJECT_SCORES_FILE="/tmp/synspam_reject"

synspam_avg_reject ()
{
> $REJECT_SCORES_FILE

grep "$DAY.*synspam\:.*reject.*" /var/log/daemon.log /var/log/daemon.log.? |cut -d'=' -f2 |cut -d' ' -f1 |while read score; do
 TOTAL_REJECT_SCORE=$(expr $TOTAL_REJECT_SCORE + $score)
echo $TOTAL_REJECT_SCORE > $REJECT_SCORES_FILE
done

echo -ne "\nNumber of connections rejected: $REJECT_COUNT\n" >> $REPORT
echo -ne "Average reject score $(expr `cat $REJECT_SCORES_FILE` / $REJECT_COUNT)\n" >> $REPORT
}

synspam_avg_accept ()
{
> $ACCEPT_SCORES_FILE

grep "$DAY.*synspam\:.*accept.*" /var/log/daemon.log /var/log/daemon.log.? |cut -d'=' -f2 |cut -d' ' -f1 |while read score; do
 TOTAL_ACCEPT_SCORE=$(expr $TOTAL_ACCEPT_SCORE + $score)
echo $TOTAL_ACCEPT_SCORE > $ACCEPT_SCORES_FILE
done

echo -ne "\nNumber of connections accepted: $ACCEPT_COUNT\n" >> $REPORT
echo -ne "Average accept score $(expr `cat $ACCEPT_SCORES_FILE` / $ACCEPT_COUNT)\n" >> $REPORT
}


as_top_ten ()
{
echo begin > /tmp/list.txt
grep "$DAY.*synspam\:.*reject.*" /var/log/daemon.log /var/log/daemon.log.? |egrep -o "[0-9]*\.[0-9]*.[0-9]*.[0-9]" >> /tmp/list.txt
echo end >> /tmp/list.txt

echo -ne "\n\nTop 10 spam sending AS\n\n" >> $REPORT

nc whois.cymru.com 43 < /tmp/list.txt |grep -v cymru | cut -d'|' -f 3 | sort -b | uniq -c | sort -r |head -10  >> $REPORT
}

> $REPORT

echo -ne "====== synspam statistics ====== \n\n" >> $REPORT

as_top_ten
synspam_avg_accept
synspam_avg_reject

if [ -f "$SASTATS" ]; then

	echo -ne "\n\n====== spamassassin statistics ===== \n\n" >> $REPORT

	if [ ! -d /tmp/sa ]; then
		mkdir /tmp/sa 
	fi

	grep "$DAY" /var/log/mail.log /var/log/mail.log.? > /tmp/sa/mail.log
	$SASTATS -l /tmp/sa -f mail.log >> $REPORT
fi

mail -s "`hostname -f` antispam report for $(date +%F --date '1 day ago')" root < $REPORT
 
